
open Format
open Instr

let print_instr fmt = function
  | Add -> fprintf fmt "add"
  | Sub -> fprintf fmt "sub"
  | Mul -> fprintf fmt "mul"
  | Div -> fprintf fmt "div"
  | Mod -> fprintf fmt "mod"
  | Not -> fprintf fmt "not"
  | Inf -> fprintf fmt "inf"
  | Infeq -> fprintf fmt "infeq"
  | Sup -> fprintf fmt "sup"
  | Supeq -> fprintf fmt "supeq"
  (* oprations sur les rels *)
  | Fadd -> fprintf fmt "fadd"
  | Fsub -> fprintf fmt "fsub"
  | Fmul -> fprintf fmt "fmul"
  | Fdiv -> fprintf fmt "fdiv"
  | Fcos -> fprintf fmt "fcos"
  | Fsin -> fprintf fmt "fsin"
  | Finf -> fprintf fmt "finf"
  | Finfeq -> fprintf fmt "finfeq"
  | Fsup -> fprintf fmt "fsup"
  | Fsupeq -> fprintf fmt "fsupeq"
  (* opration sur les adresses *)
  | Padd -> fprintf fmt "padd"
  (* opration sur les chanes *)
  | Concat -> fprintf fmt "concat"
  (* oprations sur le tas *)
  | Alloc n -> fprintf fmt "alloc %d" n
  | Allocn -> fprintf fmt "allocn"
  | Free -> fprintf fmt "free"
  (* galit *)
  | Equal -> fprintf fmt "equal"
  | Is_addr -> fprintf fmt "isaddr"
  (* conversions *)
  | Atoi -> fprintf fmt "atoi"
  | Atof -> fprintf fmt "atof"
  | Itof -> fprintf fmt "itof"
  | Ftoi -> fprintf fmt "ftoi"
  | Stri -> fprintf fmt "stri"
  | Strf -> fprintf fmt "strf"
  (* empiler *)
  | Pushi i -> fprintf fmt "pushi %d" i
  | Pushn i -> fprintf fmt "pushn %d" i
  | Pushf f -> fprintf fmt "pushf %f" f
  | Pushs s -> fprintf fmt "pushs \"%s\"" s
  | Pushg i -> fprintf fmt "pushg %d" i
  | Pushl i -> fprintf fmt "pushl %d" i
  | Pushsp -> fprintf fmt "pushsp"
  | Pushfp -> fprintf fmt "pushfp"
  | Pushgp -> fprintf fmt "pushgp"
  | Load i -> fprintf fmt "load %d" i
  | Loadn -> fprintf fmt "loadn"
  | Dup i -> fprintf fmt "dup %d" i
  | Dupn -> fprintf fmt "dupn"
  (* dpiler *)
  | Pop i -> fprintf fmt "pop %d" i
  | Popn -> fprintf fmt "popn"
  (* Stocker *)
  | Storel i -> fprintf fmt "storel %d" i
  | Storeg i -> fprintf fmt "storeg %d" i
  | Store i -> fprintf fmt "store %d" i
  | Storen -> fprintf fmt "storen" 
  (* divers *)
  | Check (i,j) -> fprintf fmt "check %d, %d" i j
  | Swap -> fprintf fmt "swap"
  (* entres-sorties *)
  | Writei -> fprintf fmt "writei"
  | Writef -> fprintf fmt "writef"
  | Writes -> fprintf fmt "writes"
  | Read -> fprintf fmt "read"
  (* primitives graphiques *)
  | DrawLine -> fprintf fmt "drawline"
  | DrawPoint -> fprintf fmt "drawpoint"
  | DrawCircle ->  fprintf fmt "drawcircle"
  | DrawRect ->  fprintf fmt "drawrect"
  | FillRect ->  fprintf fmt "fillrect"
  | SetColor ->  fprintf fmt "setcolor"
  | ClearDrawingArea -> fprintf fmt "cleardrawingarea"
  | OpenDrawingArea ->  fprintf fmt "opendrawingarea"
  | Refresh -> fprintf fmt "refresh"
  (* sauts *)
  | Label _ -> ()
  | Jump l -> fprintf fmt "jump %s" l
  | Jz l -> fprintf fmt "jz %s" l
  | Pusha l -> fprintf fmt "pusha %s" l
  (* procdures *)
  | Call -> fprintf fmt "call"
  | Return -> fprintf fmt "return"
  (* initialisation *)
  | Start -> fprintf fmt "start"
  | Nop -> fprintf fmt "nop"
  | Err s -> fprintf fmt "err \"%s\"" s
  | Stop -> fprintf fmt "stop"
  (* commentaires *)
  | Comment s -> fprintf fmt "// %s" s

let print_i fmt = function
  | Label l, "" -> fprintf fmt "%s:\n" l
  | Label l, c -> fprintf fmt "%s: // %s\n" l c
  | Comment c1, c2 -> fprintf fmt "// %s %s\n" c1 c2
  | i, "" -> fprintf fmt "\t%a\n" print_instr i
  | i, c -> fprintf fmt "\t%a // %s\n" print_instr i c

let instructions = Queue.create ()

let emit_one ?(comment="") i = Queue.push (i,comment) instructions
let emit ?comment = List.iter (emit_one ?comment)

let write_file f =
  let c = open_out f in
  let fmt = Format.formatter_of_out_channel c in
  fprintf fmt "@["; 
  Queue.iter (print_i fmt) instructions;
  fprintf fmt "@]@."; 
  close_out c
